#include "../tmc/boardAssignment.h"
#include "../tmc/idDetection.h"
#include "SelfTest.h"
#include "board.h"

static void init();
static void deInit();
static void selfTest();
static void periodicJob(uint32 tick);

IOPinTypeDef
	*groupA[SELF_TEST_PINS_PER_GROUP],
	*groupB[SELF_TEST_PINS_PER_GROUP];


EvalBoardTypeDef SelfTest = { .init = init };

static void init()
{
	groupA[0]	= &HAL.IOs->pins->DIO6;
	groupA[1]	= &HAL.IOs->pins->ID_CH1;
	groupA[2]	= &HAL.IOs->pins->DIO1;
	groupA[3]	= &HAL.IOs->pins->DIO3;
	groupA[4]	= &HAL.IOs->pins->DIO5;
	groupA[5]	= &HAL.IOs->pins->DIO8;
	groupA[6]	= &HAL.IOs->pins->DIO10;
	groupA[7]	= &HAL.IOs->pins->CLK16;
	groupA[8]	= &HAL.IOs->pins->SPI2_CSN1;
	groupA[9]	= &HAL.IOs->pins->SPI2_SCK;
	groupA[10]	= &HAL.IOs->pins->SPI2_SDI;
	groupA[11]	= &HAL.IOs->pins->SPI1_SCK;
	groupA[12]	= &HAL.IOs->pins->SPI1_SDO;
	groupA[13]	= &HAL.IOs->pins->DIO13;
	groupA[14]	= &HAL.IOs->pins->DIO15;
	groupA[15]	= &HAL.IOs->pins->DIO17;
	groupA[16]	= &HAL.IOs->pins->DIO19;

	groupB[0]	= &HAL.IOs->pins->ID_CLK;
	groupB[1]	= &HAL.IOs->pins->ID_CH0;
	groupB[2]	= &HAL.IOs->pins->DIO0;
	groupB[3]	= &HAL.IOs->pins->DIO2;
	groupB[4]	= &HAL.IOs->pins->DIO4;
	groupB[5]	= &HAL.IOs->pins->DIO7;
	groupB[6]	= &HAL.IOs->pins->DIO9;
	groupB[7]	= &HAL.IOs->pins->DIO11;
	groupB[8]	= &HAL.IOs->pins->SPI2_CSN0;
	groupB[9]	= &HAL.IOs->pins->SPI2_CSN2;
	groupB[10]	= &HAL.IOs->pins->SPI2_SDO;
	groupB[11]	= &HAL.IOs->pins->SPI1_CSN;
	groupB[12]	= &HAL.IOs->pins->SPI1_SDI;
	groupB[13]	= &HAL.IOs->pins->DIO12;
	groupB[14]	= &HAL.IOs->pins->DIO14;
	groupB[15]	= &HAL.IOs->pins->DIO16;
	groupB[16]	= &HAL.IOs->pins->DIO18;

	VitalSignsMonitor.debugMode = 1;
	EvalBoards.ch1.userFunction	= selfTest;
	EvalBoards.ch1.deInit		= deInit;
	EvalBoards.ch1.periodicJob	= periodicJob;
    EXTI_DeInit();
}

static void deInit()
{
	IdDetection.init();
	VitalSignsMonitor.debugMode = 0;
	IOMap.init();
	HAL.LEDs->error.off();
}

static void selfTest()
{
	unsigned int
		i,
		result = 0xFFFFFFFF;
	IOPinTypeDef
		**inGroup, **outGroup;


	//	1 40 1 0 0 10 C9 45 60
	//	1 40 2 0 0 0 0 0 43
	//	1 40 3 0 0 0 0 0 44
	//	1 40 5 0 0 0 0 0 45
	//	1 40 6 0 0 0 0 0 46

	//	1 40 1 0 0 10 C9 45 60 1 40 2 0 0 0 0 0 43 1 40 3 0 0 0 0 0 44

	switch(TMCL.command->Type)
	{

		case SELF_TEST_LEAVE:
			deInit();
			TMCL.reply->Status 		= REPLY_OK;
			TMCL.reply->Value.Int32 = 32168;
		break;

		case SELF_TEST_A_OUT_B_IN:
		case SELF_TEST_A_IN_B_OUT:
			inGroup 	= (TMCL.command->Type == SELF_TEST_A_OUT_B_IN) ? groupB : groupA;
			outGroup 	= (TMCL.command->Type == SELF_TEST_A_OUT_B_IN) ? groupA : groupB;
			for(i=0; i<SELF_TEST_PINS_PER_GROUP; i++)
			{
				HAL.IOs->config->toInput(inGroup[i]);
				HAL.IOs->config->toOutput(outGroup[i]);

				HAL.IOs->config->setHigh(outGroup[i]);
				if(!HAL.IOs->config->isHigh(inGroup[i])) result &= (unsigned int) (~(1<<i));

				HAL.IOs->config->setLow(outGroup[i]);
				if(HAL.IOs->config->isHigh(inGroup[i])) result &= (unsigned int) (~(1<<i));
			}
			result &= ((1<<SELF_TEST_PINS_PER_GROUP)-1);
			TMCL.reply->Status		= REPLY_OK;
			TMCL.reply->Value.Int32	= result;
		break;

		case SELF_TEST_READ_AN:

			TMCL.reply->Status 		= REPLY_OK;
			TMCL.reply->Value.Int32 = TMCL.command->Motor;
			switch(TMCL.command->Motor)
			{
				case 0:	TMCL.reply->Value.Int32 = (unsigned int) (((*HAL.ADCs->AIN0)/4095.0)*5.016*10);	break;
				case 1:	TMCL.reply->Value.Int32 = (unsigned int) (((*HAL.ADCs->AIN1)/4095.0)*5.016*10);	break;
				case 2:	TMCL.reply->Value.Int32 = (unsigned int) (((*HAL.ADCs->AIN2)/4095.0)*5.016*10);	break;
				case 3:	TMCL.reply->Value.Int32 = VitalSignsMonitor.VM;	break;
				default:
					TMCL.reply->Status = REPLY_CMD_NOT_AVAILABLE;
				break;
			}
		break;

		case SELF_TEST_SET_AN:
			TMCL.reply->Status 		= REPLY_OK;
			TMCL.reply->Value.Int32 = TMCL.command->Motor;
			switch(TMCL.command->Motor)
			{
				case 0:
					HAL.IOs->config->reset(&HAL.IOs->pins->AIN0);

					HAL.IOs->config->toOutput(&HAL.IOs->pins->AIN1);
					HAL.IOs->config->toOutput(&HAL.IOs->pins->AIN2);

					HAL.IOs->config->setHigh(&HAL.IOs->pins->AIN1);
					HAL.IOs->config->setHigh(&HAL.IOs->pins->AIN2);
				break;

				case 1:
					HAL.IOs->config->reset(&HAL.IOs->pins->AIN1);

					HAL.IOs->config->toOutput(&HAL.IOs->pins->AIN0);
					HAL.IOs->config->toOutput(&HAL.IOs->pins->AIN2);

					HAL.IOs->config->setHigh(&HAL.IOs->pins->AIN0);
					HAL.IOs->config->setHigh(&HAL.IOs->pins->AIN2);
				break;

				case 2:
					HAL.IOs->config->reset(&HAL.IOs->pins->AIN2);

					HAL.IOs->config->toOutput(&HAL.IOs->pins->AIN0);
					HAL.IOs->config->toOutput(&HAL.IOs->pins->AIN1);

					HAL.IOs->config->setHigh(&HAL.IOs->pins->AIN0);
					HAL.IOs->config->setHigh(&HAL.IOs->pins->AIN1);
				break;

				default:
					TMCL.reply->Status = REPLY_CMD_NOT_AVAILABLE;
				break;
			}

		break;


		case SELF_TEST_SET_AN_2:
			TMCL.reply->Status 		= REPLY_OK;
			TMCL.reply->Value.Int32 = TMCL.command->Motor;
			switch(TMCL.command->Motor)
			{
				case 0:
					HAL.IOs->config->reset(&HAL.IOs->pins->AIN0);

					HAL.IOs->config->toOutput(&HAL.IOs->pins->AIN1);
					HAL.IOs->config->toOutput(&HAL.IOs->pins->AIN2);

					HAL.IOs->config->setHigh(&HAL.IOs->pins->AIN1);
					HAL.IOs->config->setLow(&HAL.IOs->pins->AIN2);
				break;

				case 1:
					HAL.IOs->config->reset(&HAL.IOs->pins->AIN1);

					HAL.IOs->config->toOutput(&HAL.IOs->pins->AIN0);
					HAL.IOs->config->toOutput(&HAL.IOs->pins->AIN2);

					HAL.IOs->config->setHigh(&HAL.IOs->pins->AIN0);
					HAL.IOs->config->setLow(&HAL.IOs->pins->AIN2);
				break;

				case 2:
					HAL.IOs->config->reset(&HAL.IOs->pins->AIN2);

					HAL.IOs->config->toOutput(&HAL.IOs->pins->AIN0);
					HAL.IOs->config->toOutput(&HAL.IOs->pins->AIN1);

					HAL.IOs->config->setHigh(&HAL.IOs->pins->AIN0);
					HAL.IOs->config->setLow(&HAL.IOs->pins->AIN1);
				break;

				default:
					TMCL.reply->Status = REPLY_CMD_NOT_AVAILABLE;
				break;
			}

		break;

		default:
			TMCL.reply->Status = REPLY_WRONG_TYPE;
			return;
		break;
	}

}

static void periodicJob(uint32 tick)
{
	static unsigned int lastTick = 0;

	if(abs(tick-lastTick) >= 500)
	{
		HAL.LEDs->error.toggle();
		lastTick = tick;
	}
}
