#include "../../derivative.h"
#include "../../HAL.h"

#define AIRCR_VECTKEY_MASK    ((uint32)0x05FA0000)

static void init(void);
static void reset(uint8 ResetPeripherals);
static void NVIC_DeInit(void);

static const IOsFunctionsTypeDef IOFunctions =
{
	.config = &IOs,
	.pins	= &IOMap,
	.LOW	= IOS_LOW,
	.HIGH	= IOS_HIGH,
	.OPEN	= IOS_OPEN
};

const HALTypeDef HAL =
{
	.init			= init,
	.reset			= reset,
	.NVIC_DeInit 	= NVIC_DeInit,
	.SPI			= &SPI,
	.USB			= &USB,
	.LEDs			= &LEDs,
	.ADCs			= &ADCs,
	.IOs 			= &IOFunctions,
	.RS232 			= &RS232,
	.Wireless		= &Wireless,
	.SystemTick		= &SystemTick,
	.Timer			= &Timer
};

static void init(void)
{
	__enable_irq();
	IOs.init();
	IOMap.init();
	SystemTick.init();
	USB.init();
	SPI.init();
	Wireless.init();
	RS232.init();
	LEDs.init();
	ADCs.init();
}

static void reset(uint8 ResetPeripherals)
{
	if(ResetPeripherals)
	{
		SCB->AIRCR = AIRCR_VECTKEY_MASK | (u32)0x04;
	}
	else
	{
		SCB->AIRCR = AIRCR_VECTKEY_MASK | (u32)0x01;
	}
}

static void NVIC_DeInit(void)
{
	uint32 index;

	for(index=0; index<8; index++)
	{
		NVIC->ICER[index] = 0xFFFFFFFF;
		NVIC->ICPR[index] = 0xFFFFFFFF;
	}

	for(index = 0; index < 240; index++)
	{
		 NVIC->IP[index] = 0x00000000;
	}
}

void _exit(int i)
{
	UNUSED(i);
}

void _kill(void)
{
}

void _getpid(void)
{
}
